#!/bin/bash -ex

DB_NAME=	# Plesk Billing standalone datatabse name
DB6_NAME=	# Plesk Billing bundle database name
ADMIN_PASSWORD=`cat /etc/psa/.psa.shadow` || die "Unable to read database password"
PLESK_DB_NAME="psa"

# define $INSTALL_PREFIX properly
INSTALL_PREFIX="/opt"
[ "FreeBSD" = `uname -s` ] && INSTALL_PREFIX="/usr/local"

PPB_CONFIG_FILE="$INSTALL_PREFIX/plesk-billing/lib-billing/include/config/config.php"

echo2()
{
	echo "$@" >&2
}

die()
{
	echo2 "Error: $@"
	exit 1
}

usage()
{
	die "Usage: $0 -d databasename"
}

escape_str()
{
	# converts ' --> \'  and  \ --> \\
	echo "$1" | sed -e 's|\(['\''\]\)|\\\1|g'	# for vim syntax - '
}

db_query()
{
	local query="$1";
	local database="${2:-$DB_NAME}"
	mysql -u admin "-p$ADMIN_PASSWORD" -A --skip-column-names --batch --raw -e "$query" "$database"	# MySQL 3.23 doesn't recognize --skip-auto-rehash option, used -A instead
}

# parse command line arguments and define parameters for the database (v5.6) connection
define_db_params()
{
	while getopts u:p:d: opt
	do
		case "$opt" in
			d)	DB_NAME="$OPTARG";;
			\?)	usage;;
		esac
	done
	shift `expr $OPTIND - 1`

	if [ -z "$DB_NAME" ]; then
		usage
	fi

	# check database connection
	mysqladmin status -u admin "-p$ADMIN_PASSWORD" > /dev/null || die "Unable to connect to MySQL"

	# check if database exists
	db_query "SELECT NOW()" > /dev/null || die "Database '$DB_NAME' does not exist"

	# check if database belongs to standalone Plesk Billing
	query="SELECT config_param_value FROM config_params WHERE config_param_name='installVersion'";
	version=`db_query "$query"`

	case "$version" in
		5.3*);;
		5.6*);;
		6.0*);;
		*) die "Incompatible database version: '$version'";;
	esac

	# define current database name
	begin="<?php ini_set('include_path', '.'); error_reporting(0); require_once('$PPB_CONFIG_FILE'); echo \$CONF['dbName'];?>"
	DB6_NAME=`echo $begin | /usr/local/psa/admin/bin/php` || die "Unable to define current database name"

	[ -z "$DB6_NAME" ] && die "Unable to define current database name"

	if [ "$DB_NAME" = "$DB6_NAME" ]; then
		die "Database name you supplied as a command line parameter is the same as in configuration file '$PPB_CONFIG_FILE'"
	fi

	# check if database exists
	db_query "SELECT NOW()" "$DB6_NAME" > /dev/null || die "Database '$DB6_NAME' (from Plesk Billing cobfiguration file) does not exist"
}

# backup database
backup_database()
{
	mysqldump -u admin "-p$ADMIN_PASSWORD" "$DB_NAME" > "./$DB_NAME.dump.`date +%Y-%m-%d-%T`.sql" || die "Unable to back up database."
}

# modify database (5.6)
convert_database()
{
	# define id of the 'internal' config group
	config_group_id=`db_query "SELECT config_group_id FROM config_groups WHERE config_group_name='internal'"` || die "Unable to define ID for 'internal' config group"

	db_query "UPDATE config_params SET config_param_value='plesk' WHERE config_group_id='$config_group_id' AND config_param_name='license'" || die "Unable to change license type"
}

# change PPB connection info in Plesk
set_ppb_connection()
{
	# define clients ID for admin user
	client_id=`db_query 'SELECT id FROM clients WHERE type="admin" LIMIT 1' "$PLESK_DB_NAME"` || die "Unable to define client_id for admin"

	# set up admin_remote_access_hash
	admin_remote_access_hash=`db_query 'SELECT admin_remote_access_hash FROM admins WHERE admin_username="admin_api"'` || die "Unable to define admin_remote_access_hash"
	admin_remote_access_hash=`escape_str "$admin_remote_access_hash"`

	db_query "REPLACE INTO cl_param VALUES($client_id, 'remote-access-hash', '$admin_remote_access_hash')" "$PLESK_DB_NAME" || die "Unable to set up admin_remote_access_hash"
}


# copy (move?) SSO registration info from the original PPB 6.0 database
copy_sso_registration()
{
	# remove old records
	query="DELETE FROM config_params WHERE config_params.config_group_id=3 AND config_params.config_param_name LIKE 'app-modernbill-%'"
	db_query "$query" || die "Unable to copy SSO registration info"

	# copy SSO info
	query="INSERT INTO \`$DB_NAME\`.config_params SELECT * FROM \`$DB6_NAME\`.config_params WHERE \`$DB6_NAME\`.config_params.config_group_id=3 AND \`$DB6_NAME\`.config_params.config_param_name LIKE 'app-modernbill-%'"
	db_query "$query" || die "Unable to copy SSO registration info"
}

# grant access to modified database
grant_db_access()
{
	# define current database username
	begin="<?php ini_set('include_path', '.'); error_reporting(0); require_once('$PPB_CONFIG_FILE'); echo \$CONF['dbUser'];?>"
	DB_USERNAME=`echo $begin | /usr/local/psa/admin/bin/php` || die "Unable to define current database username"

	# grant access
	db_query "GRANT ALL ON \`$DB_NAME\`.* TO '$DB_USERNAME'@'localhost'" || die "Unable to grant access to database"
	db_query "FLUSH PRIVILEGES" || die "Unable to FLUSH PRIVILEGES"
}

# changes database name in PPB's config file
change_db_name()
{
	sed -e "s/^\s*\\\$CONF\[['\"]dbName['\"]\]\s*=.*$/\$CONF[\"dbName\"] = \"$DB_NAME\";/g" < "$PPB_CONFIG_FILE" > "$PPB_CONFIG_FILE.tmp" && \
	mv "$PPB_CONFIG_FILE.tmp" "$PPB_CONFIG_FILE" || die "Unable to change database name in Plesk Billing configuration file"
}

###################### MAIN PROGRAM STARTS HERE ###############

define_db_params "$@"

backup_database

convert_database

set_ppb_connection

copy_sso_registration

grant_db_access

change_db_name
