<?php
/*
 * This is example script that demonstrates how an application (SP, service Provider)
 * can be attached to (or detached from) SSO server (IdP, identity provider)
 *
 * Enjoy
 * -- Wally, the best Dilbert's friend.
 */

require_once('common_header.php');


function makeUnexpectedCodeMessage(HTTP_Request $request) {
    echo "URL:", $request->getUrl(), "\n";
    return "Unexpected result code "
            . $request->getResponseCode() . " returned from IdP. Response body:\n"
            . $request->getResponseBody();
}


/**
 * Generator of HTTP requests featuring SSL certificates authentication.
 * SSL certificates information are placed PEM-encoded in $client_cert_file and $server_pub_key_file
 * @param $method Name of HTTP method
 * @param $url Request URL
 * @param $client_cert_file string
 * @param $server_pub_key_file string
 * @return HTTP_Request
 */
function createSecureHttpRequest($method, $url, $client_cert_file, $server_pub_key_file) {
    $options = array(
        'timeout' => 30,
        'saveBody' => true
    );
    // Setup SSL context
    $options['socketOptions'] = array(
        'ssl' => array(
            'local_cert'  => $client_cert_file,
            'passphrase'  => '',
            'verify_peer' => true,
            'cafile'      => $server_pub_key_file,
            'allow_self_signed' => true,
            // 'capath'  => '',
            // 'CN_match' => 'peer-name'
        ));
    $request = new HTTP_Request($url, $options);
    $request->setMethod($method);
    return $request;
}


function doSecureHttpRequest($method, $url, array $props, array $headers, $body) {
    // Unfortuanately we can not read from string (damn PHP). Need to save to file.
    $client_cert_tf = 'client_cert.tmp';
    file_put_contents($client_cert_tf,
        base64_decode($props['sp_cert']) . "\n"
        . base64_decode($props['sp_pkey']));
    $server_pub_key_tf = 'idp_pub_key.tmp';
    file_put_contents($server_pub_key_tf,
        base64_decode($props['idp_cert']));

    $req = createSecureHttpRequest(
            $method,
            $url,
            $client_cert_tf,
            $server_pub_key_tf );
    foreach ($headers as $name => $val) {
        $req->addHeader($name, $val);
    }
    $req->setBody($body);
    $req->sendRequest();

    unlink($client_cert_tf);
    unlink($server_pub_key_tf);
    return $req;
}


function doSecureSpRequest($method, $reg_name, $headers, $body) {
    $idp_props = getRegProps($reg_name);
    if (!$idp_props) {
        throw new Exception("No registration properties found for '$reg_name'");
    }
    return doSecureHttpRequest(
        $method,
        $idp_props['api_url'] . '/sp/' . $idp_props['sp_id'] . '/props',
        $idp_props,
        $headers,
        $body );
}


/*
 * Register SP in IdP
 */
function registerSp($idp_url, $reg_name) {
    if (getRegProps($reg_name)) {
        echo "'$reg_name' is already registered.\n";
        return 1;
    }
    $req = new HTTP_Request($idp_url . '/sp');
    $req->setMethod('POST');

    $reg_props = getRegRequestProps($reg_name);
    foreach ($reg_props as $key => $val) {
        $req->addPostData($key, $val);
    }
    $req->sendRequest();

    $idp_response = $req->getResponseBody();
    if ($req->getResponseCode() != 201) {
        echo makeUnexpectedCodeMessage($req);
        return 1;
    } else {
        putRegProps($reg_name, lib_Props::parse($idp_response));
        file_put_contents(getIdpRegFile($reg_name), $idp_response);
        echo "Registered $reg_name\n";
        return 0;
    }
}


/*
 * Update registration information.
 */
function updateSpRegistration($reg_name) {
    $reg_props = getRegRequestProps($reg_name);
    $reg_props['reissue_cert'] = 'true'; // optional
    $req = doSecureSpRequest(
        'PUT',
        $reg_name,
        array('Content-Type' => 'text/plain'),
        lib_Props::toString($reg_props) );
    $response_code = $req->getResponseCode();
    $idp_response = $req->getResponseBody();
    if ($response_code == 200) {
        file_put_contents(getIdpRegFile($reg_name), $idp_response);
        echo "Updated registration.\n";
        return 0;
    } else {
        echo makeUnexpectedCodeMessage($req);
        return 1;
    }
}


/*
 * Remove registration information from IdP
 */
function detachSp($reg_name) {
    // Remove both registration entries (own and other's party)
    $req = doSecureSpRequest('DELETE', $reg_name, array(), '');
    $response_code = $req->getResponseCode();
    if ($response_code == 200) {
        unlink(getIdpRegFile($reg_name));
        echo "Unlinked from IdP.\n";
        return 0;
    } else {
        echo makeUnexpectedCodeMessage($req);
        return 1;
    }
}


function printUsage() {
    echo <<< EOX
SP registration demo script.
Usage:
    php-cli register_sp.php register registration_name idp_url
    php-cli register_sp.php detach registration_name
    php-cli register_sp.php update registration_name
Where
    register
    detach
    update
        Action to perform
    registration_name
        Base name of file that holds (or shoud hold) registration data.
        Registration request properties should be in registration_name.reg-props file.
        Registration result properties are stored in registration_name.idp-reg file.
    idp_url
        Base URL of exposed IdP API
Example:
    php-cli register_sp.php register example https://127.0.0.1:11443

EOX
;
}


function main($argv) {
    if (count($argv) < 3) {
        printUsage();
        return 1;
    } else {
        $action = $argv[1];
        $sp_reg_name = $argv[2];
        switch ($action) {
            case 'register':
                if (count($argv) != 4) {
                    printUsage();
                    return 1;
                }
                $idp_url = $argv[3];
                $ret = registerSp($idp_url, $sp_reg_name);
                return $ret;
            case 'detach':
                $ret = detachSp($sp_reg_name);
                return $ret;
            case 'update':
				$ret = updateSpRegistration($sp_reg_name);
                return $ret;
            default:
                echo "Unknown action '$action'\n";
                printUsage();
                return 1;
        }
    }
}

if ($_SERVER['argc'] > 0) { // If running from command line
    $res = main($_SERVER['argv']);
    // file_put_contents(__FILE__.'.deps', implode("\n", get_included_files()));
    exit($res);
}

?>